import React, { Suspense, useEffect, useState } from 'react';
import { RootErrorBoundary } from './components/ErrorBoundary/RootErrorBoundary';
import { AppErrorBoundary } from './components/ErrorBoundary/AppErrorBoundary';
import { OfflineStatusBar } from './components/Offline/OfflineStatusBar';
import { OfflineIndicator } from './components/Offline/OfflineIndicator';
import { FeedbackButton } from './components/BetaFeedback/FeedbackButton';
import { DemoControls } from './components/DemoMode/DemoControls';
import { DiagnosticsPanel } from './components/Admin/DiagnosticsPanel';
import { Routes } from './Routes';
import { featureFlags } from './lib/feature-flags';
import { GlobalLoader } from './components/LoadingStates/GlobalLoader';
import { useAuthStore } from './store/auth';
import { usePreferencesStore } from './store/preferences';
import { logger } from './lib/logging';
import { useLocation, useNavigate } from 'react-router-dom';
import { ageVerification } from './lib/age-verification';
import { useRouteMonitor } from './lib/hooks';
import { Toaster } from './components/ui/toaster';
import { AgeCheckModal } from './components/Auth/AgeCheckModal';
import { PreviewModeBar } from './components/Auth/PreviewModeBar';
import { AuthErrorToast } from './components/Auth/AuthErrorToast';
import { AuthErrorBoundary } from './components/ErrorBoundary/AuthErrorBoundary';
import { testSupabaseConnection } from './lib/supabase-test';
import { setupSupabaseErrorLogging } from './lib/supabase-logger';
import { Helmet, HelmetProvider } from 'react-helmet-async';
import { PointsManager } from './components/Points/PointsManager';
import { scheduleWeeklyReset, checkWeeklyReset } from './lib/points-utils';
import { usePointsStore } from './store/points';
import { OnboardingTour } from './components/Onboarding/OnboardingTour';
import { tourManager, TourId } from './lib/tour-manager';
import { DailyStreakReminder } from './components/StreakReminder/DailyStreakReminder';

// Global error handler
if (typeof window !== 'undefined') {
  window.onerror = (message, source, lineno, colno, error) => {
    logger.error('Global error:', {
      message,
      source,
      lineno,
      colno,
      error,
    });
  };

  window.onunhandledrejection = (event) => {
    logger.error('Unhandled promise rejection:', {
      reason: event.reason,
      promise: event.promise,
    });
  };
}

export const App = () => {
  const { initializeAuth, user } = useAuthStore();
  const { fetchPreferences } = usePreferencesStore();
  const { resetWeeklyStreak } = usePointsStore();
  const location = useLocation();
  const navigate = useNavigate();
  const [showMainTour, setShowMainTour] = useState(false);
  
  // Monitor route changes
  useRouteMonitor();

  useEffect(() => {
    // Diagnostic logging for Supabase environment variables
    logger.info('Supabase URL:', import.meta.env.VITE_SUPABASE_URL ? 'Present' : 'Missing');
    logger.info('Supabase Anon Key:', import.meta.env.VITE_SUPABASE_ANON_KEY ? 'Present (First 5 chars): ' + 
      (import.meta.env.VITE_SUPABASE_ANON_KEY?.substring(0, 5) + '...') : 'Missing');

    // Setup Supabase error logging
    setupSupabaseErrorLogging();
    
    // Run Supabase connection test in development
    if (import.meta.env.DEV) {
      testSupabaseConnection()
        .then(result => {
          logger.info('Supabase connection test complete:', result);
        })
        .catch(err => {
          logger.error('Supabase connection test failed:', err);
        });
    }

    const initializeApp = async () => {
      try {
        logger.info('[App] Initializing auth and preferences');
        await initializeAuth();
        await fetchPreferences();

        // Check age verification on initial load
        if (!ageVerification.isVerified() && 
            location.pathname !== '/age-gate' && 
            location.pathname !== '/auth' &&
            location.pathname !== '/terms' && 
            location.pathname !== '/privacy' &&
            location.pathname !== '/verify-email' &&
            location.pathname !== '/reset-password') {
          navigate('/auth', { replace: true });
        }

        // Check if we need to reset weekly streaks
        if (checkWeeklyReset()) {
          await resetWeeklyStreak();
        }

        // Schedule weekly streak reset
        scheduleWeeklyReset();

        // Check if this is the first visit to show main tour
        if (tourManager.isNewUser()) {
          tourManager.markUserVisited();
          setShowMainTour(true);
        }

        logger.info('[App] Initialization complete');
      } catch (error) {
        logger.error('Failed to initialize app:', error as Error);
      }
    };

    initializeApp();

    return () => {
      logger.info('[App] Component unmounting');
    };
  }, [initializeAuth, fetchPreferences, navigate, location.pathname, resetWeeklyStreak]);

  // Main app tour steps
  const mainTourSteps = tourManager.getTourSteps(TourId.MAIN);

  return (
    <HelmetProvider>
      <Helmet>
        <title>Clouded - Cannabis Rankings & Discovery</title>
        <meta name="description" content="Discover and rate the best cannabis products in Las Vegas" />
        <meta name="viewport" content="width=device-width, initial-scale=1.0" />
        <meta name="theme-color" content="#581C87" />
      </Helmet>
      <RootErrorBoundary>
        <AppErrorBoundary>
          <AuthErrorBoundary>
            <PointsManager>
              <div className="min-h-screen bg-gradient-to-r from-purple-900 via-purple-800 to-purple-900">
                <Suspense fallback={<GlobalLoader />}>
                  <Routes />
                  <OfflineStatusBar />
                  <OfflineIndicator />
                  <FeedbackButton />
                  {featureFlags.isEnabled('DEMO_MODE') && <DemoControls />}
                  <DiagnosticsPanel />
                  <Toaster />
                  <AgeCheckModal />
                  <PreviewModeBar />
                  <AuthErrorToast />
                  <DailyStreakReminder />
                  
                  {/* Main App Tour */}
                  {showMainTour && (
                    <OnboardingTour 
                      tourId={TourId.MAIN}
                      steps={mainTourSteps}
                      onComplete={() => {
                        setShowMainTour(false);
                        tourManager.markTourCompleted(TourId.MAIN);
                      }}
                      onSkip={() => {
                        setShowMainTour(false);
                        tourManager.markTourCompleted(TourId.MAIN);
                      }}
                    />
                  )}
                </Suspense>
              </div>
            </PointsManager>
          </AuthErrorBoundary>
        </AppErrorBoundary>
      </RootErrorBoundary>
    </HelmetProvider>
  );
};